#!/bin/sh
#
# Experiment replication script for the TACAS 2018 paper
# "A Statistical Model Checker for Nondeterminism and Rare Events"
# by Carlos E. Budde, Pedro R. D'Argenio, Arnd Hartmanns and Sean Sedwards
#
# Replicating results of Table 1: "Performance and scalability on the electric vehicle charging case study"
#
# 1. Run Monte Carlo simulation with one thread (row "1", columns "MC" for "n_fail = 2" and "n_fail = 3").
#    Higher values for n_fail are not included because of expected runtime > 10 minutes.
#    Expected results, in file vehicle-hpn.mc.replicated-1.txt:
#    (a) Confidence intervals that overlap with the ones reported in row "conf. interval" in the paper.
#    (b) Simulation times around 20% above the values reported in the paper on host systems similar to ours.
#    Expected total runtime: around 40 seconds.
#
echo
echo Experiment 1 / 10
echo Model:         vehicle-hpn.modest
echo Configuration: Monte Carlo, 1 thread
echo Output file:   vehicle-hpn.mc.replicated-1.txt
echo Expected time: \< 1 minute
echo
mono Modest/modes.exe Models/vehicle-hpn.modest -R Uniform -S ASAP -J 1 -E "MIN_FAILURES=2" -E "MIN_FAILURES=3" --statistical CI --relative-width -O vehicle-hpn.mc.replicated-1.txt -Y
#
# 2. Run Monte Carlo simulation with two threads (row "2", columns "MC" for "n_fail = 2", "n_fail = 3" and "n_fail = 4").
#    Expected results, in file vehicle-hpn.mc.replicated-2.txt:
#    (a) Confidence intervals that overlap with the ones reported in row "conf. interval" in the paper.
#    (b) Simulation times about 30% below the ones for 1 thread on a multi-core host system.
#        Since the VM is configured with only 2 cores, the speedup is lower than the one reported between 1 and 2 threads in the paper.
#    Expected total runtime: around 8.5 minutes.
#
echo
echo Experiment 2 / 10
echo Model:         vehicle-hpn.modest
echo Configuration: Monte Carlo, 2 threads
echo Output file:   vehicle-hpn.mc.replicated-2.txt
echo Expected time: \< 10 minutes
echo
mono Modest/modes.exe Models/vehicle-hpn.modest -R Uniform -S ASAP -J 2 -E "MIN_FAILURES=2" -E "MIN_FAILURES=3" -E "MIN_FAILURES=4" --statistical CI --relative-width -O vehicle-hpn.mc.replicated-2.txt -Y
#
# 3. Run rare event simulation with RESTART and one thread (row "1", columns "RES" for "n_fail = 2", "n_fail = 3" and "n_fail = 4").
#    Expected results, in file vehicle-hpn.res.replicated-1.txt:
#    (a) Confidence intervals that overlap with the ones reported in row "conf. interval" in the paper.
#    (b) Simulation times lower than the ones for Monte Carlo simulation with 1 thread for n_fail >= 3.
#    Expected total runtime: below 10 minutes, with high variance.
#
echo
echo Experiment 3 / 10
echo Model:         vehicle-hpn.modest
echo Configuration: RESTART, 1 thread
echo Output file:   vehicle-hpn.res.replicated-1.txt
echo Expected time: \< 10 minutes
echo
mono Modest/modes.exe Models/vehicle-hpn.modest -R Uniform -S ASAP -J 1 -E "MIN_FAILURES=2" -E "MIN_FAILURES=3" -E "MIN_FAILURES=4" --statistical CI --relative-width --rare Restart -O vehicle-hpn.res.replicated-1.txt -Y
#
# 4. Run Monte Carlo simulation with two threads (row "2", columns "MC" for "n_fail = 2", "n_fail = 3" and "n_fail = 4").
#    Expected results, in file vehicle-hpn.res.replicated-2.txt:
#    (a) Confidence intervals that overlap with the ones reported in row "conf. interval" in the paper.
#    (b) Simulation times lower than the ones for Monte Carlo simulation with 2 threads for n_fail >= 3.
#    Expected total runtime: below 5 minutes, with high variance.
#
echo
echo Experiment 4 / 10
echo Model:         vehicle-hpn.modest
echo Configuration: RESTART, 2 threads
echo Output file:   vehicle-hpn.res.replicated-2.txt
echo Expected time: \< 5 minutes
echo
mono Modest/modes.exe Models/vehicle-hpn.modest -R Uniform -S ASAP -J 2 -E "MIN_FAILURES=2" -E "MIN_FAILURES=3" -E "MIN_FAILURES=4" --statistical CI --relative-width --rare Restart -O vehicle-hpn.res.replicated-2.txt -Y
#
#
# Replicating results of Table 2: "Performance and results for the low-latency wireless network case study"
#
# In order to reduce the runtime for the artifact evaluation, we only sample 10 schedulers instead of 100 as in the paper. As a result, the obtained maximum and minimum estimates will more often be further from the optimal values than the ones reported in the paper.
#
# 1. Run Monte Carlo simulation with LSS and one thread (row "1", but for only 10 schedulers).
#    Expected results, in file token-wireless.lss-10-mc.replicated-1.txt:
#    (a) Minimum and maximum estimates that are (up to statistical outliers) contained within the interval spanned by the actual minimum and maximum values reported in the paper in row "optimal".
#    (b) Simulation time of around 500 seconds on host systems similar to ours.
#    Expected total runtime: around 8.5 minutes.
#
echo
echo Experiment 5 / 10
echo Model:         token-wireless.modest
echo Configuration: Monte Carlo / LSS, 1 thread
echo Output file:   token-wireless.lss-10-mc.replicated-1.txt
echo Expected time: \< 10 minutes
echo
mono Modest/modes.exe Models/token-wireless.modest --unsafe -M MDP --digital-clocks --width 0.0025 -J 1 -L 10 -O token-wireless.lss-10-mc.replicated-1.txt -Y
#
# 2. Run Monte Carlo simulation with LSS and two threads (row "2", but for only 10 schedulers).
#    Expected results, in file token-wireless.lss-10-mc.replicated-2.txt:
#    (a) Minimum and maximum estimates that are (up to statistical outliers) contained within the interval spanned by the actual minimum and maximum values reported in the paper in row "optimal".
#    (b) Simulation time about 30% below the one for 1 thread on a multi-core host system.
#        Since the VM is configured with only 2 cores, the speedup is lower than the one reported between 1 and 2 threads in the paper.
#    Expected total runtime: around 6 minutes.
#
echo
echo Experiment 6 / 10
echo Model:         token-wireless.modest
echo Configuration: Monte Carlo / LSS, 2 threads
echo Output file:   token-wireless.lss-10-mc.replicated-2.txt
echo Expected time: \< 10 minutes
echo
mono Modest/modes.exe Models/token-wireless.modest --unsafe -M MDP --digital-clocks --width 0.0025 -J 2 -L 10 -O token-wireless.lss-10-mc.replicated-2.txt -Y
#
#
# Replicating results of Table 3: "Performance and results for the reliable database system case study"
#
# Parameter R is constant "RED" in the model.
#
# 1. Run Monte Carlo simulation with the uniform scheduler and two threads (rows with R = "2" and "3", columns "MC" and "conf. interval" under heading "uniform scheduler").
#    Higher values for R are not included because of expected runtime much greater than 10 minutes.
#    Expected results, in file database.uni-mc.replicated-2.txt:
#    (a) Confidence intervals that overlap the ones reported in the paper.
#    (b) Simulation times of around 3 seconds for R = 2 and around 450 seconds for R = 3 on host systems similar to ours.
#    Expected total runtime: around 7.5 minutes.
#
echo
echo Experiment 7 / 10
echo Model:         database.modest
echo Configuration: Monte Carlo / uniform, 2 threads
echo Output file:   database.uni-mc.replicated-2.txt
echo Expected time: \< 10 minutes
echo
mono Modest/modes.exe Models/database.modest --unsafe -M MA -E "RED=2" -E "RED=3" -R Uniform --statistical CI --relative-width -J 2 -O database.uni-mc.replicated-2.txt -Y
#
# 2. Run rare event simulation with RESTART, the uniform scheduler, and two threads (rows with R = "2", "3" and "4", columns "RES" and "conf. interval" under heading "uniform scheduler").
#    Higher values for R are not included because of expected runtime greater than 10 minutes.
#    Expected results, in file database.uni-restart.replicated-2.txt:
#    (a) Confidence intervals that overlap the ones reported in the paper.
#    (b) Simulation times significantly below those for Monte Carlo simulation when R >= 3 on host systems similar to ours.
#    Expected total runtime: below 5 minutes, with high variance.
#
echo
echo Experiment 8 / 10
echo Model:         database.modest
echo Configuration: RESTART / uniform, 2 threads
echo Output file:   database.uni-restart.replicated-2.txt
echo Expected time: \< 5 minutes
echo
mono Modest/modes.exe Models/database.modest --unsafe -M MA -E "RED=2" -E "RED=3" -E "RED=4" -R Uniform --statistical CI --relative-width --rare Restart --ifun-op Max Max -J 2 -O database.uni-restart.replicated-2.txt -Y
#
# 3. Run Monte Carlo simulation with LSS and two threads (row with R = "2", columns "MC", "min. conf. int." and "max. conf. int." under heading "lightweight scheduler sampling (20)").
#    Higher values for R are not included because of expected runtime greater than 10 minutes.
#    Expected results, in file database.lss-20-mc.replicated-2.txt:
#    (a) Maximum and minimum confidence intervals that overlap the ones reported in the paper.
#    (b) Simulation time of around 70 seconds on host systems similar to ours.
#    Expected total runtime: around 1.5 minutes.
#
echo
echo Experiment 9 / 10
echo Model:         database.modest
echo Configuration: Monte Carlo / LSS, 2 threads
echo Output file:   database.lss-20-mc.replicated-2.txt
echo Expected time: \< 2 minutes
echo
mono Modest/modes.exe Models/database.modest --unsafe -M MA -E "RED=2" -L 20 --statistical CI --relative-width -J 2 -O database.lss-20-mc.replicated-2.txt -Y
#
# 4. Run rare event simulation with RESTART, LSS, and two threads (rows with R = "2" and "3", columns "RES", "min. conf. int." and "max. conf. int." under heading "lightweight scheduler sampling (20)").
#    Higher values for R are not included because of expected runtime much greater than 10 minutes.
#    Expected results, in file database.uni-restart.replicated-2.txt:
#    (a) Confidence intervals that overlap the ones reported in the paper. In particular, for R = 3, the min. and max. confidence intervals should not overlap.
#    (b) Simulation times of around 350 seconds for R = 2 and 500 seconds for R = 3 on host systems similar to ours.
#    Expected total runtime: below 15 minutes, with high variance.
#
echo
echo Experiment 10 / 10
echo Model:         database.modest
echo Configuration: RESTART / LSS, 2 threads
echo Output file:   database.lss-20-restart.replicated-2.txt
echo Expected time: \< 15 minutes
echo
mono Modest/modes.exe Models/database.modest --unsafe -M MA -E "RED=2" -E "RED=3" -L 20 --statistical CI --relative-width --rare Restart --ifun-op Max Max -J 2 -O database.lss-20-restart.replicated-2.txt -Y
